@@ -8,7 +8,7 @@ use XML::Parser::PerlSAX;
 use XML::Handler::Dtd2Html;
 
 my %opts;
-getopts('bCfMs:t:o:Z', \%opts);
+getopts('bCfMs:t:o:x:Z', \%opts);
 
 my $handler = new XML::Handler::Dtd2Html();
 my $parser = new XML::Parser::PerlSAX(Handler => $handler, ParseParamEnt => 1);
@@ -33,13 +33,17 @@ if (exists $opts{o}) {
 	$outfile = "dtd_" . $root;
 }
 
+my @examples = ();
+@examples = split /\s+/, $opts{x}
+		if (exists $opts{x});
+
 if      ($opts{b}) {
 	bless($doc, "XML::Handler::Dtd2Html::DocumentBook");
 } elsif ($opts{f}) {
 	bless($doc, "XML::Handler::Dtd2Html::DocumentFrame");
 }
 
-$doc->generateHTML($outfile, $opts{t}, $opts{s}, !exists($opts{C}), exists($opts{M}), exists($opts{Z}));
+$doc->generateHTML($outfile, $opts{t}, $opts{s}, \@examples, !exists($opts{C}), exists($opts{M}), exists($opts{Z}));
 
 __END__
 
@@ -49,7 +53,7 @@ dtd2html - Generate a HTML documentation from a DTD
 
 =head1 SYNOPSYS
 
-dtd2html [B<-b> | B<-f>] [B<-C> | B<-M>] [B<-Z>] [B<-o> I<filename>] [B<-s> I<style>] [B<-t> I<title>] I<xml_file>
+dtd2html [B<-b> | B<-f>] [B<-C> | B<-M>] [B<-Z>] [B<-o> I<filename>] [B<-s> I<style>] [B<-t> I<title>] [B<-x> 'I<example1.xml> I<example2.xml> ...'] I<file.xml>
 
 =head1 OPTIONS
 
@@ -83,6 +87,10 @@ Generate an external I<style>.css file.
 
 Specify the title of the resulting HTML file.
 
+=item -x
+
+Include a list of XML files as examples.
+
 =item -Z
 
 Delete zombi element (without parent).
@@ -111,11 +119,47 @@ This tool needs XML::Parser::PerlSAX (libxml-perl) and XML::Parser modules.
 
 XML::Parser::PerlSAX v0.07 needs to be patched (PerlSAX.patch).
 
+=head2 Comments
+
+XML files (and DTD) can include comments. The XML syntax is :
+
+ <!-- comments -->
+
+All comments before a declaration are captured (except with -M option).
+Each comment generates its own paragraph E<lt>pE<gt>.
+
+=head2 Standard HTML
+
+You can embed standard HTML tags within a comment. However, don't use tags
+heading tags like E<lt>h1E<gt> or E<lt>hrE<gt>. B<dtd2html> creates an entire
+structured document and these structural tags interfere with formatting of
+the generated document.
+
+So you must use entities &lt; &gt; &amp; within a comment.
+
+=head2 dtd2html Tags
+
+B<dtd2html> parses tags that are recognized when they are embedded
+within an XML comment. These doc tags enable you to autogenerate a
+complete, well-formatted document from your XML source. The tags start with
+an @.
+
+Tags must start at the beginning of a line.
+
+The special tag @INCLUDE allows inclusion of the content of an external file.
+
+ <!--
+   comments
+   @Version : 1.0
+   @INCLUDE : description.txt
+   @See Also : REC-xml
+ -->
+
 =head1 SEE ALSO
 
 PerlSAX.pod(3), XML::Handler::Dtd2Html
 
- Extensible Markup Language (XML) <http://www.w3c.org/TR/REC-xml>
+Extensible Markup Language (XML), E<lt>http://www.w3c.org/TR/REC-xmlE<gt>
 
 =head1 AUTHOR
 
@@ -123,6 +167,8 @@ Francois PERRAD, francois.perrad@gadz.org
 
 =head1 COPYRIGHT
 
+(c) 2002 Francois PERRAD, France. All rights reserved.
+
 This program is distributed under the Artistic License.
 
 =cut
